package org.jboss.qa.brms.persistencequery;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.command.Command;
import org.drools.command.CommandFactory;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.ExecutionResults;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.QueryResults;
import org.drools.runtime.rule.QueryResultsRow;
import org.drools.runtime.rule.Variable;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class PersistenceQueryTest {

    public static void main(String[] args) {
        KnowledgeBase kbase = newBase();
        StatefulKnowledgeSession ksession = newSession(kbase);

        List<Command<?>> commands = new ArrayList<Command<?>>();
        commands.add(CommandFactory.newInsert(new Fact("first", "second")));
        commands.add(CommandFactory.newInsert(new Fact("second", "third")));
        commands.add(CommandFactory.newInsert(new Fact("third", "fourth")));
        commands.add(CommandFactory.newInsert(new Fact("fourth", "fifth")));

        commands.add(CommandFactory.newFireAllRules());

        commands.add(CommandFactory.newQuery("query", "follows", new Object[] { "first", Variable.v}));

        ExecutionResults execResults = ksession.execute(CommandFactory.newBatchExecution(commands));

        QueryResults results = (QueryResults)execResults.getValue("query");
        for (QueryResultsRow result : results) {
            System.out.println(result.get("next"));
        }
        ksession.dispose();

        System.out.println("All finished...");
    }

    private static KnowledgeBase newBase() {
        System.setProperty("java.naming.factory.initial", "bitronix.tm.jndi.BitronixInitialContextFactory");

        PoolingDataSource ds = new PoolingDataSource();
        ds.setUniqueName( "jdbc/BitronixJTADataSource" );
        ds.setClassName( "bitronix.tm.resource.jdbc.lrc.LrcXADataSource" );
        ds.setMaxPoolSize( 3 );
        ds.setAllowLocalTransactions(true);
        ds.getDriverProperties().put("driverClassName", "org.hsqldb.jdbcDriver");
        ds.getDriverProperties().put("url", "jdbc:hsqldb:file:/tmp/testDB");
        ds.init();

        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newClassPathResource("ruleset.drl", PersistenceQueryTest.class), ResourceType.DRL);

        if (kbuilder.hasErrors()) {
            throw new RuntimeException(kbuilder.getErrors().toString());
        }

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

        return kbase;
    }

    private static StatefulKnowledgeSession newSession(KnowledgeBase kbase) {
        KnowledgeSessionConfiguration ksconf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();

        EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.drools.persistence.jpa");
        Environment env = KnowledgeBaseFactory.newEnvironment();
        env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
        env.set(EnvironmentName.TRANSACTION_MANAGER, TransactionManagerServices.getTransactionManager());

        return JPAKnowledgeService.newStatefulKnowledgeSession(kbase, ksconf, env);
    }
}
